%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This script developed by Nico, serves to predict spindle (or division) orientation based on
% cell shape. The hypothesis is that the mitotic figure that specify the
% division plane is oriented by Microtubule radiating from two centrosomes that generate 
% pulling forces that scale  to microtubule length (see Minc et al. Cell 2011, for details).

% Getting started: Place in a folder images .tif that contain cells with
% particular shapes that you want to analyze, run the script. The script
% will interact with the user and output an energy landscape that informs
% on the likeliness of dividing at a given angle. 

% Output: For each picture analyzed a folder is created containing: (i) a stack
% tiff file is saved to display the main results (Image of the cell and
% prediction of spindle orientation)
% and (ii) a .mat file with all saved variables. 

%As examples, the package provides a couple of images of dividiongcell to test the script. 
% Enjoy!! 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%


close all
clear all
clc

pro=dir('*.tif');

for K=5:length(pro),  
 

%% These are the parameters you can play with

N=21; %  MT Number par aster  (put an odd number)
anglelimit=2/4; % how far the aster extends arounds the centrosomes, 1/2 corresponds to -90 90
Beta=3; % Exponent for force scaling as a function of length for single MTs
Spindlesize=1/3; % Factor to scale spindle length to cell size (1 mean spindle has the same length as the cell diameter). 


%%


display(K)    
filenamer= [ pro(K).name ]; 
Images(K).image= imread(filenamer);
Im(K).im=mat2gray(Images(K).image);    
figure;
imshow(Im(K).im);

 prompt = {'Can you position centrosomes manually? Yes=1, No=0'};
                    def = {'1'};
                    num_lines = 1;
                    R = inputdlg(prompt,'Q',num_lines,def);
                    R=str2num(R{:});


 prompt = {'How many cells do you want to study in this image?'};
                    def = {'1'};
                    num_lines = 1;
                    P = inputdlg(prompt,'Q',num_lines,def);
                    P=str2num(P{:});

title('');   
mkdir(['Cells_',num2str(K)]);

close;        

for i=1:P,
  figure; 
  if R==1,
imshow(Im(K).im,[],'InitialMagnification',500); title(['Manually trace the contour of cell #',num2str(i),' and place 2 points, 1 for each centrosome']);
Sman=size(Im(K).im); 
Pman=impoly(gca,[]);
apiman=iptgetapi(Pman);
pointsman(i).points=apiman.getPosition(); 
BWfinal(i).bwf= poly2mask(pointsman(i).points(:,1),pointsman(i).points(:,2), Sman(1), Sman(2));

   
CentroExp1=impoint(gca,[]);
apiCentro1 = iptgetapi(CentroExp1);
pointCentro1(i).pointcentro1=apiCentro1.getPosition();
CentroExp2=impoint(gca,[]);
apiCentro2 = iptgetapi(CentroExp2);
pointCentro2(i).pointcentro2=apiCentro2.getPosition();

  else

   
imshow(Im(K).im,[],'InitialMagnification',500); title(['Manually trace the contour of cell #',num2str(i)']);
Sman=size(Im(K).im); 
Pman=impoly(gca,[]);
apiman=iptgetapi(Pman);
pointsman(i).points=apiman.getPosition(); 
BWfinal(i).bwf= poly2mask(pointsman(i).points(:,1),pointsman(i).points(:,2), Sman(1), Sman(2));
      
  end     
      
end
    
    close all; 
    
 for i=1:P,
     
% Label the white part in the BW image
Dim(i).dim = size(BWfinal(i).bwf);
Label(i).label=bwlabel(BWfinal(i).bwf);
Stat(i)=regionprops(Label(i).label,'Centroid','Orientation','Area','MajorAxisLength','MinorAxisLength');

% tracer outilne vert
BWoutline(i).bwout = bwperim(BWfinal(i).bwf);
[Row(i).row, Col(i).col]=find(BWoutline(i).bwout,1);
connectivity=8;
num_points=360;
Contour(i).contour = bwtraceboundary(BWfinal(i).bwf, [Row(i).row, Col(i).col], 'N', connectivity);

% Find the centre of mass
Centre(i).centre=Stat(i).Centroid;
surface(i).surface=Stat(i).Area; 
longax(i).longax=Stat(i).MajorAxisLength;
shortax(i).shortax=Stat(i).MinorAxisLength;
TotAscpectratio(i)=longax(i).longax/shortax(i).shortax;
Xc(i)=Centre(i).centre(1);
Yc(i)=Centre(i).centre(2);
scaleforce(i)=((sqrt(surface(i).surface/pi))^Beta)*N;


Row(i).row = round(Yc(i));
Col(i).col = find(BWfinal(i).bwf(Row(i).row,:), 1,'last');
Diststart(i).diststart=max(Dim(i).dim(1), Dim(i).dim(2));


if R==1,
Xcexp(i)=(pointCentro1(i).pointcentro1(1)+ pointCentro2(i).pointcentro2(1))/2;
Ycexp(i)=(pointCentro1(i).pointcentro1(2)+ pointCentro2(i).pointcentro2(2))/2;
DistanceInCenter(i)=pi*(((Xcexp(i)-Xc(i))^2+ (Ycexp(i)-Yc(i))^2));


% Coordonees exp des centrosomes et orientation du fuseau
if pointCentro2(i).pointcentro2(2)==pointCentro1(i).pointcentro1(2)
Signe(i)=1;
else
Signe(i)=sign(pointCentro2(i).pointcentro2(2)- pointCentro1(i).pointcentro1(2));
end

ExpSpindleaxis(i)= sqrt((pointCentro1(i).pointcentro1(1)-pointCentro2(i).pointcentro2(1))^2+ (pointCentro1(i).pointcentro1(2)-pointCentro2(i).pointcentro2(2))^2);
expOrientation(i)=180/pi*acos(-Signe(i)*(pointCentro2(i).pointcentro2(1)-pointCentro1(i).pointcentro1(1))/ExpSpindleaxis(i));
Errorincentering(i)=abs(DistanceInCenter(i)./surface(i).surface);
ThSpindleLegnth(i).thSpindleLegnth=(ExpSpindleaxis(i));

else
% 
  ThSpindleLegnth(i).thSpindleLegnth=2*Spindlesize*sqrt(surface(i).surface)/pi();
 
  end 
 
 disp(i);

% Turn around the image and position all possible avis and centrosomes
% positions
for j=1:181,

 Alpha(i).alpha(j)=pi/180*((j-1));
 Alphaindeg(i).alphaindeg(j)=180/pi*Alpha(i).alpha(j);
 
 % Position of the centrosomes
 absCentro1(i).absCentro1(j)=Xc(i)+ ThSpindleLegnth(i).thSpindleLegnth/2*cos(Alpha(i).alpha(j));
 ordCentro1(i).ordCentro1(j)=Yc(i)- ThSpindleLegnth(i).thSpindleLegnth/2*sin(Alpha(i).alpha(j));
   
 absCentro2(i).absCentro2(j)=Xc(i)- ThSpindleLegnth(i).thSpindleLegnth/2*cos(Alpha(i).alpha(j));
 ordCentro2(i).ordCentro2(j)=Yc(i)+ ThSpindleLegnth(i).thSpindleLegnth/2*sin(Alpha(i).alpha(j)); 
        
 %trace MTs fro each angle
   
 for k=1:N+1,
      
    Abscfromcentro1(i).abscfromcentro1(j,k)=absCentro1(i).absCentro1(j)+ Diststart(i).diststart*cos(pi*((k-1)/N-1/2)+pi/(180)*(j-1));
    Ordofromcentro1(i).ordofromcentro1(j,k)= ordCentro1(i).ordCentro1(j)- Diststart(i).diststart*sin(pi*((k-1)/N-1/2)+pi/(180)*(j-1));
    
    Abscfromcentro2(i).abscfromcentro2(j,k)=absCentro2(i).absCentro2(j)+ Diststart(i).diststart*cos(pi*((k-1)/N+1/2)+pi/(180)*(j-1));
    Ordofromcentro2(i).ordofromcentro2(j,k)=ordCentro2(i).ordCentro2(j)- Diststart(i).diststart*sin(pi*((k-1)/N+1/2)+pi/(180)*(j-1));
    
    angleMT1(i).angleMT1(k)= pi*((k-1)/N-1/2)*2*anglelimit;
    angleMT2(i).angleMT2(k)= pi*(2*anglelimit*(k-1)/N+(1-anglelimit));
    
 % Microtubule lengths in micrometer.
    profile1(i).profile1 = improfile( BWfinal(i).bwf,[absCentro1(i).absCentro1(j) Abscfromcentro1(i).abscfromcentro1(j,k)],[ordCentro1(i).ordCentro1(j) Ordofromcentro1(i).ordofromcentro1(j,k)], Diststart(i).diststart);
    profile2(i).profile2 = improfile( BWfinal(i).bwf,[absCentro2(i).absCentro2(j) Abscfromcentro2(i).abscfromcentro2(j,k)],[ordCentro2(i).ordCentro2(j) Ordofromcentro2(i).ordofromcentro2(j,k)], Diststart(i).diststart);
    MTlength1(i).MTlength1(j,k) = find(profile1(i).profile1==1, 1, 'last');  
    MTlength2(i).MTlength2(j,k) = find(profile2(i).profile2==1, 1, 'last');
    
 % Torques and force for each MTs
    torqueperMT1(i).torqueperMT1(j,k)= (MTlength1(i).MTlength1(j,k))^(Beta)*sin(angleMT1(i).angleMT1(k));   
    torqueperMT2(i).torqueperMT2(j,k)= - (MTlength2(i).MTlength2(j,k))^(Beta)*sin(angleMT2(i).angleMT2(k));  
    
    forceperMT1(i).forceperMT1(j,k)=(MTlength1(i).MTlength1(j,k))^(Beta)*cos(angleMT1(i).angleMT1(k));
    forceperMT2(i).forceperMT2(j,k)=-(MTlength2(i).MTlength2(j,k))^(Beta)*cos(angleMT2(i).angleMT2(k));
    
                    end
   
    % Renormalized Total torque, force and potential from the 2 aster: 
    
        
    torqueaster(i).torqueaster(j)=sum(torqueperMT1(i).torqueperMT1(j,:)+ torqueperMT2(i).torqueperMT2(j,:))/scaleforce(i);    
    potential(i).potential(j)=-sum(torqueaster(i).torqueaster(1:j));
    Totalforce(i).totalforce(j)=sum(forceperMT1(i).forceperMT1(j,:)+ forceperMT2(i).forceperMT2(j,:))/scaleforce(i);
       
end

 for j=1:181,
potential(i).potential(j)=potential(i).potential(j)-(potential(i).potential(181)-potential(i).potential(1))*j/180;
 end
    Maxindex(i)=find( potential(i).potential == min( potential(i).potential));
    
 for j=1:181,
      Normalizedpot(i).normalizedpot(j)=(potential(i).potential(j)-max(potential(i).potential));
 end
 end
 
 
  %% Plots Figure with all relevant information and comparaison
 cd (['Cells_',num2str(K)]);
 for i=1:P,
 if R==1,
ExpPotential(i)=Normalizedpot(i).normalizedpot(round(expOrientation(i)+1));
DeltaAngle(i)=abs(Maxindex(i)-(expOrientation(i)+1));  
 
if DeltaAngle(i) > 90,
DeltaAngle(i)=180-DeltaAngle(i);
      
    else
DeltaAngle(i)=DeltaAngle(i);
end
      
  
figplot=figure('Position',[100 100 1200 400]) ; 
 subplot(1,3,1);
imshow(Im(K).im); hold; 
 
plot(Contour(i).contour(:,2),Contour(i).contour(:,1),'r','LineWidth',2); 
plot( absCentro1(i).absCentro1(Maxindex(i)),ordCentro1(i).ordCentro1(Maxindex(i)),'o'); plot( absCentro2(i).absCentro2(Maxindex(i)),ordCentro2(i).ordCentro2(Maxindex(i)),'o');
line([absCentro1(i).absCentro1(Maxindex(i)) absCentro2(i).absCentro2(Maxindex(i))], [ordCentro1(i).ordCentro1(Maxindex(i)) ordCentro2(i).ordCentro2(Maxindex(i))],'color','g', 'LineWidth',2);
subplot(1,3,2);; plot(Alphaindeg(i).alphaindeg , Normalizedpot(i).normalizedpot,'k'); hold; plot(round(expOrientation(i)), ExpPotential(i),'+'); plot(Alphaindeg(i).alphaindeg(Maxindex(i)),Normalizedpot(i).normalizedpot(Maxindex(i)),'o'); 
title('Potential'),set(gca,'XTick',0:45:180);

MaxPot=max(ExpPotential);
subplot(1,3,3); bar( 100*DeltaAngle/90); axis([0.5 P+0.5 0 100]); title('Orienting Deviation Btw Model and experiment');

g=getframe(figplot); imwrite(g.cdata,['Results_Cells',num2str(K),'.tif'],'WriteMode', 'append', 'Compression','none');
 
 else  
figplot=figure('Position',[100 100 1200 600]) ; 

subplot(1,2,1);
imshow(Im(K).im); hold; 
plot(Contour(i).contour(:,2),Contour(i).contour(:,1),'r','LineWidth',2); 
plot( absCentro1(i).absCentro1(Maxindex(i)),ordCentro1(i).ordCentro1(Maxindex(i)),'o'); plot( absCentro2(i).absCentro2(Maxindex(i)),ordCentro2(i).ordCentro2(Maxindex(i)),'o');
line([absCentro1(i).absCentro1(Maxindex(i)) absCentro2(i).absCentro2(Maxindex(i))], [ordCentro1(i).ordCentro1(Maxindex(i)) ordCentro2(i).ordCentro2(Maxindex(i))],'color','g', 'LineWidth',2);

subplot(1,2,2); plot(Alphaindeg(i).alphaindeg , Normalizedpot(i).normalizedpot,'k'); hold; plot(Alphaindeg(i).alphaindeg(Maxindex(i)),Normalizedpot(i).normalizedpot(Maxindex(i)),'o'); 
title('Potential'),set(gca,'XTick',0:45:180);

g=getframe(figplot); imwrite(g.cdata,['Results_Cells',num2str(K),'.tif'],'WriteMode', 'append', 'Compression','none');
  
 end 
 end
 
%% Save all info 
     save (['workspace_Cells_',num2str(K),'.mat']);
    cd ..
  clear all
  close all
  clc
  pro=dir('*.tif');
 end 


